@@ -1,5 +1,3 @@
-## create a named cell grob
-## a function of an integer index that returns a named grob
 
 textii <- function(d, gp=gpar(), name="row-label-", just="center"){
   x <- switch(just, "center"=0.5, "right"=0.95, "left"=0.1)
@@ -50,7 +48,7 @@ makeTableGrobs <- function(content, rnames=NULL, cnames=NULL,
  gp.rowfille <- gp.rowfillo <- gpar.rowfill
  gp.rowfille[["alpha"]] <-  h.even.alpha
  gp.rowfillo[["alpha"]] <-  h.odd.alpha
-
+## print(nrow)
  gpar.rowfill <- rep(c(list(gp.rowfille), list(gp.rowfillo)), nrow)
  
  gp.colfille <- gp.colfillo <- gpar.colfill
@@ -109,9 +107,11 @@ makeTableGrobs <- function(content, rnames=NULL, cnames=NULL,
 
 
 arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
-                               padding.h = unit(4, "mm"), padding.v=unit(4, "mm"), 
-                               just=c("center", "center"), separator= "white",
-                               show.box=FALSE, show.csep=FALSE, show.rsep=FALSE){
+                              show.colnames, show.rownames, 
+                              padding.h = unit(4, "mm"), padding.v=unit(4, "mm"), 
+                              just=c("center", "center"), separator= "white",
+                              show.vlines=FALSE, show.hlines=FALSE, show.namesep=FALSE, 
+                              show.box=FALSE, show.csep=FALSE, show.rsep=FALSE){
  
   label.ind <- 1   # index running accross labels
 
@@ -128,24 +128,39 @@ arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
     }
   }
 
-        
-  ## draw horizontal lines, stopping or not before the names
-  for (ii in seq(1, nrow, 1)) 
+  if(show.hlines){
+    ## draw horizontal lines, stopping or not before the names
+    for (ii in seq(2, nrow, 1)) 
       grid.segments(0, 0, 1, 0, gp=gpar(col=separator), vp=viewport( layout.pos.row=ii,
-                                 layout.pos.col=seq(1+!show.rsep, ncol+1)))
-  ## draw vertical lines, stopping or not before the names
-  for (jj in seq(1, ncol, 1)) 
-      grid.segments(1, 0, 1, 1, gp=gpar(col=separator), vp=viewport( layout.pos.col=jj,
-                                 layout.pos.row=seq(1+!show.csep, nrow+1)))
-  ## draw boxes around the content and the names
-  if(show.box){
-  grid.rect(gp=gpar(col=separator),
-            vp=viewport( layout.pos.col=seq(2, ncol+1), layout.pos.row=seq(2, nrow+1)))
-  grid.rect(gp=gpar(col=separator), 
-            vp=viewport( layout.pos.col=seq(2, ncol+1), layout.pos.row=1))
-  grid.rect(gp=gpar(col=separator),
-            vp=viewport( layout.pos.row=seq(2, nrow+1), layout.pos.col=1))
+                                                          layout.pos.col=seq(1+!show.rsep, ncol+1)))
   }
+  if(show.vlines){
+    ## draw vertical lines, stopping or not before the names
+    for (jj in seq(2, ncol, 1)) 
+      grid.segments(1, 0, 1, 1, gp=gpar(col=separator), vp=viewport( layout.pos.col=jj,
+                                                          layout.pos.row=seq(1+!show.csep, nrow+1)))
+  }
+  ## draw box around the content (and the names if present)
+  if(show.box){
+    if(show.colnames & show.rownames){
+       grid.border(type=16, colour=separator,
+            vp=viewport( layout.pos.col=seq(2, ncol+1), layout.pos.row=seq(2, nrow+1)))
+     } else 
+    grid.rect(gp=gpar(col=separator), 
+                vp=viewport( layout.pos.col=seq(1, ncol+1), layout.pos.row=seq(1, nrow+1)))
+  }
+  
+  if(show.namesep){
+  ## draw corner, style depends on the presence of row/colnames
+    type <- if(show.colnames & show.rownames) 8 else
+    if(show.colnames & !show.rownames) 4 else
+    if(!show.colnames & show.rownames) 5 else 1
+    
+      grid.border(type=type, colour=separator,
+            vp=viewport( layout.pos.col=seq(2, ncol+1), layout.pos.row=seq(2, nrow+1)))
+  }
+  
+
 }
 
 
@@ -161,20 +176,23 @@ arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
 ##' @param col.just justification of labels
 ##' @param core.just justification of labels
 ##' @param separator colour of the border lines 
-##' @param show.box logical
-##' @param show.csep logical separator for colnames 
-##' @param show.rsep logical separator for rownames 
-##' @param equal.width logical  
+##' @param show.box logical box surrounding the table
+##' @param show.vlines logical vertical lines
+##' @param show.hlines logical horizontal lines
+##' @param show.namesep logical draw lines to separate header(s)
+##' @param show.csep logical extend vert. separator to colnames 
+##' @param show.rsep logical extend vert. separator to rownames 
+##' @param equal.width logical 
 ##' @param equal.height logical  
-##' @param padding.h unit of horizontal margin,  per cell
-##' @param padding.v unit of vertical margin,  per cell
+##' @param padding.h unit of horizontal margin, per cell
+##' @param padding.v unit of vertical margin, per cell
 ##' @param gpar.coretext gpar() for inner text
 ##' @param gpar.corefill gpar() for inner fill
 ##' @param gpar.coltext gpar() for colnames text
-##' @param h.odd.alpha h.odd.alpha 
-##' @param h.even.alpha h.even.alpha 
-##' @param v.odd.alpha v.odd.alpha 
-##' @param v.even.alpha v.even.alpha 
+##' @param h.odd.alpha numeric transparency factor for odd horizontal cells
+##' @param h.even.alpha numeric transparency factor for even horizontal cells
+##' @param v.odd.alpha numeric transparency factor for odd vertical cells
+##' @param v.even.alpha numeric transparency factor for even vertical cells
 ##' @param gpar.colfill gpar() for colnames fill
 ##' @param gpar.rowtext gpar() for rownames text
 ##' @param gpar.rowfill gpar() for rownames fill
@@ -184,9 +202,8 @@ arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
 ##' @param theme theme (list of aesthetic elements)
 ##' @param ... passed to grob
 ##' @return a grob of class table
+##' 
 ##' @examples
-##' 
-##' 
 ##' grid.table(head(iris), h.even.alpha=1, h.odd.alpha=1,  v.even.alpha=0.5, v.odd.alpha=1)
 ##' grid.newpage()
 ##' grid.draw(tableGrob(head(iris, 10), name="test"))
@@ -202,7 +219,7 @@ arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
 ##' lg <- lapply(c("theme.blank", "theme.default", "theme.white",  "theme.vertical",  "theme.list", "theme.black"),
 ##'              function(x) tableGrob(head(iris[, 1:3]), theme=get(x)()))
 ##' grid.newpage()
-##' do.call(arrange, lg)
+##' do.call(grid.arrange, lg)
 ##' \dontrun{
 ##' ## timing: a bit slow due to repeated on-the-fly calculations 
 ##' pdf("test2.pdf", height=50)
@@ -214,7 +231,9 @@ arrangeTableGrobs <- function(lgt, lgf, nrow, ncol, widths, heights,
 tableGrob <- function(d, rows=rownames(d), cols=colnames(d),
                       show.rownames=TRUE, show.colnames=TRUE,
                       row.just="center", col.just="center", core.just="center", 
-                      separator="white", show.box=FALSE, show.csep=FALSE, show.rsep=FALSE,
+                      separator="white", show.box=FALSE,
+                      show.vlines=FALSE, show.hlines=FALSE, show.namesep=FALSE, 
+                      show.csep=FALSE, show.rsep=FALSE,
                       equal.width = FALSE, equal.height=FALSE, 
                       padding.h = unit(4, "mm"), padding.v=unit(4, "mm"),
                       gp=NULL, 
@@ -235,7 +254,7 @@ tableGrob <- function(d, rows=rownames(d), cols=colnames(d),
   lg <- 
   with(theme, 
        makeTableGrobs(as.character(as.matrix(d)), rows, cols,
-                        nrow(d), ncol(d),
+                        NROW(d), NCOL(d),
                         row.just = row.just, col.just = col.just, core.just = core.just, 
                         equal.width = equal.width, equal.height = equal.height, 
                         gpar.coretext = gpar.coretext,
@@ -252,6 +271,7 @@ tableGrob <- function(d, rows=rownames(d), cols=colnames(d),
          show.rownames=show.rownames, show.colnames=show.colnames,
          row.just = row.just, col.just = col.just, core.just = core.just, 
          separator=separator, show.box=show.box,
+         show.vlines=show.vlines, show.hlines=show.hlines, show.namesep=show.namesep, 
          show.csep=show.csep, show.rsep=show.rsep,
          equal.width = equal.width, equal.height = equal.height, 
          padding.h = padding.h, padding.v = padding.v, 
@@ -273,9 +293,9 @@ grid.table <- function(...)
 
 
 drawDetails.table <- function(x, recording=TRUE){
-  
+
   lg <- with(x, makeTableGrobs(as.character(as.matrix(d)), rows, cols,
-         nrow(d), ncol(d),
+         NROW(d), NCOL(d),
          row.just = row.just, col.just = col.just, core.just = core.just, 
          equal.width = equal.width, equal.height = equal.height, 
          gpar.coretext = gpar.coretext,
@@ -301,9 +321,12 @@ drawDetails.table <- function(x, recording=TRUE){
   
   pushViewport(cells)
   tg <- arrangeTableGrobs(lg$lgt, lg$lgf, lg$nrow, lg$ncol, lg$widths, lg$heights,
-          padding.h = x$padding.h, padding.v = x$padding.v, 
-          separator=x$separator, show.box=x$show.box,
-          show.csep=x$show.csep, show.rsep=x$show.rsep)
+                          show.colnames=x$show.colnames, show.rownames=x$show.rownames, 
+                          padding.h = x$padding.h, padding.v = x$padding.v, 
+                          separator=x$separator, show.box=x$show.box,
+                          show.vlines=x$show.vlines, show.hlines=x$show.hlines,
+                          show.namesep=x$show.namesep, 
+                          show.csep=x$show.csep, show.rsep=x$show.rsep)
   upViewport()
 }
 widthDetails.table <- function(x){
@@ -341,19 +364,20 @@ updatelist <- function (x, y)
 ##' @return theme
 theme.default <- theme.grey <- function(...)
   updatelist(list(show.rownames=TRUE, show.colnames=TRUE,
-     row.just="center", col.just="center", core.just="center", 
-     separator="white", show.box=FALSE, show.csep=FALSE, show.rsep=FALSE,
-     equal.width = FALSE, equal.height=FALSE, 
-     padding.h = unit(4, "mm"), padding.v=unit(4, "mm"),
-     gp=NULL, 
-     gpar.coretext = gpar(col="black", cex=1),
-     gpar.coltext =  gpar(col="black", cex=1, fontface="bold"),
-     gpar.rowtext =  gpar(col="black", cex=0.8, fontface="italic"),
-     h.odd.alpha = 1, h.even.alpha = 1, 
-     v.odd.alpha = 1, v.even.alpha = 1, 
-     gpar.corefill = gpar(fill = "grey95", col="white"), 
-     gpar.rowfill = gpar(fill = "grey90", col="white"), 
-     gpar.colfill = gpar(fill = "grey90", col="white")), list(...))
+                  row.just="center", col.just="center", core.just="center", 
+                  separator="white", show.box=FALSE, show.vlines=FALSE, show.hlines=FALSE, show.namesep=FALSE,
+                  show.csep=FALSE, show.rsep=FALSE,
+                  equal.width = FALSE, equal.height=FALSE, 
+                  padding.h = unit(4, "mm"), padding.v=unit(4, "mm"),
+                  gp=NULL, 
+                  gpar.coretext = gpar(col="black", cex=1),
+                  gpar.coltext =  gpar(col="black", cex=1, fontface="bold"),
+                  gpar.rowtext =  gpar(col="black", cex=0.8, fontface="italic"),
+                  h.odd.alpha = 1, h.even.alpha = 1, 
+                  v.odd.alpha = 1, v.even.alpha = 1, 
+                  gpar.corefill = gpar(fill = "grey95", col="white"), 
+                  gpar.rowfill = gpar(fill = "grey90", col="white"), 
+                  gpar.colfill = gpar(fill = "grey90", col="white")), list(...))
 
 
 theme.list <- function(...)
